require 'test_helper'

class KanbanFlowTest < ActionController::IntegrationTest
  fixtures :all
  
  def setup
    Task.delete_all #タスクとユーザーを初期化します
    User.delete_all  
  end

  test "login and task flow" do #テストの名前
    post_via_redirect "/users/create", #ユーザーを登録します
      :user => {
        :name => "Tanaka", 
        :password => "pass", 
        :password_confirmation => "pass"}
    assert_response :success #レスポンスのステータスを確認
    assert_equal '/users', path #ユーザー一覧にredirectされたことを確認
    assert_equal(1, User.find(:all).size) #ユーザーが1件登録されたことを確認

    get "/tasks" #タスク一覧に遷移
    assert_response :success
    assert_template "index" #indexテンプレートが描画されたことを確認

    get "/users/login" #ログイン画面に遷移
    assert_response :success
    assert_template "login"

    post_via_redirect "/users/login", #先ほど登録したユーザーでログインを行う
      :name => "Tanaka", 
      :password => "pass"
    assert_response :success
    assert_equal '/tasks', path #/tasks に最後にリクエストが送信されたことを確認
    assert_equal "Tanaka", session[:user] #sessionにユーザーが格納されたことを確認
     

    
    post_via_redirect "/tasks/create", #タスクを3件作成する
      :task => {
        :content => "ToDo Task", #statusはすべてToDo。1件目
        :status =>  "ToDo",
        :priority => 1,
        :owner => "Tanaka"}
    post_via_redirect "/tasks/create",
      :task => {
        :content => "Doing Task", #2件目
        :status =>  "ToDo",
        :priority => 2,
        :owner => "Ono"}
    post_via_redirect "/tasks/create",
      :task => {
        :content => "Done Task", #3件目
        :status =>  "ToDo",
        :priority => 3,
        :owner => "Yamaya"}
    assert_equal(3, Task.find(:all).size)  #3件登録されたことを確認


    tasks = Task.find(:all, :conditions => ["content = ? ", "Doing Task"] ) 
    assert_equal(1, tasks.size) #Doingにするタスクを選んで件数を確認する

    post "/tasks/change_status", #選択したタスクの状態をDoingに変更する
      :id => tasks[0].id,
      :status => "Doing"
    assert_response :success

    tasks = Task.find(:all, :conditions => ["content = ? ", "Done Task"] )
    assert_equal(1, tasks.size) #同様にDoneにするタスクを選んで件数を確認する

    post "/tasks/change_status", #選択したタスクの状態をDoneに変更する
      :id => tasks[0].id,
      :status => "Done"
    assert_response :success

    get "/tasks" #タスク一覧画面を再取得する
    tasks = assigns(:tasks) #@tasksインスタンス変数をassignsで取り出す
    assert_equal 3, tasks.size #tasksには状態ごとのモデルが格納されているので3件

    assert_equal(1, tasks["ToDo"].size) #各状態が1件ずつになっていることを確認
    assert_equal(1, tasks["Doing"].size) 
    assert_equal(1, tasks["Done"].size) 

    post_via_redirect "/users/logout" #ログアウトの処理を行う
    assert_response :success
    assert_equal 'ログアウトしました', flash[:notice] #flashメッセージを確認する
  end
  
  
  test "dual session task update" do #テストの名前
    post_via_redirect "/users/create", #ユーザーを2人登録します、1人目
      :user => {
        :name => "Tanaka", 
        :password => "pass", 
        :password_confirmation => "pass"}
    post_via_redirect "/users/create", #ユーザー登録2人目
      :user => {
        :name => "Ono", 
        :password => "pass", 
        :password_confirmation => "pass"}
    assert_equal(2, User.find(:all).size) #ユーザーが2件登録されたことを確認

    post_via_redirect "/tasks/create", #タスクを1件作成する
      :task => {
        :content => "content1", #statusはToDo
        :status =>  "ToDo",
        :priority => 1,
        :owner => "Tanaka"}
    assert_equal(1, Task.find(:all).size)  #1件登録されたことを確認

    user = {:name => "Ono", :password => "pass"}
    ono = user_login(user) #一番下に書いてあるprivateメソッドを呼び出し、
    user = {:name => "Tanaka", :password => "pass"}
    tanaka = user_login(user) #新しいセッションを2つ開始する

    tasks = Task.find(:all) 

    ono.change_task_status(tasks[0], "Doing") #タスクの状態をDoingに変更
    tanaka.change_task_status(tasks[0], "Done") #別のユーザーでタスクの状態をDoneに変更

    get "/tasks" 
    tasks = assigns(:tasks) 
    assert_equal(0, tasks["ToDo"].size) 
    assert_equal(0, tasks["Doing"].size) 
    assert_equal(1, tasks["Done"].size) #Done1件だけになっていることを確認
  end
  
  private
  def user_login(user)
    open_session do |sess|
      sess.https!
      sess.post "/login", :username => user[:name], :password => user[:password]
      sess.https!(false)
      def sess.change_task_status(task, status)
        post "/tasks/change_status", #選択したタスクの状態をDoneに変更する
          :id => task.id,
          :status => status
        assert_response :success
      end
    end
  end
end
