require 'test_helper'

class TaskTest < ActiveSupport::TestCase
  fixtures :tasks

  test "string attributes are not empty" do
    t = Task.new(:priority => 0)
    assert !t.valid?
    assert t.errors.invalid?(:content)
    assert t.errors.invalid?(:status)
    assert t.errors.invalid?(:owner)
  end
  
  test "priority range 0 to 3" do
    good = (0..3)
    bad = %W{ -20 -1 4 100 }
    
    good.each do |v|
      t = Task.new(:content=>"do something", 
        :status=>"ToDo", :priority=>v, :owner=>"田中")
      assert t.valid?, t.errors.full_messages
    end
    
    bad.each do |v|
      t = Task.new(:content=>"do nothing", 
        :status=>"Done", :priority=>v, :owner=>"小野")
      assert !t.valid?, "priority:#{v}"
    end
  end
  
  test "should content is unique" do
    t = Task.new(:content => tasks(:make_kanban).content,
      :status=>"ToDo", :priority=>1, :owner=>"小野")
    assert !t.save
  end
end
